#region Copyright  2004-2006 Victor Boctor @ http://www.futureware.biz
//
// TaskVisionToMantis is copyrighted to Victor Boctor
//
// This program is distributed under the terms and conditions of the GPL
// See LICENSE file for details.
//
// http://www.futureware.biz
//
#endregion

using System;
using System.Collections;
using System.Data;
using System.Data.SqlClient;

namespace TaskVisionToMantis
{
	/// <summary>
	/// A class that handles the migration from Task Vision to Mantis Bugtracker.
	/// </summary>
	public class TaskVisionToMantis
	{
		private const string TaskVisionConnectionString = "Data Source=MyServerName;Initial Catalog=TaskVision;Integrated Security=SSPI;";
		private const string MantisConnectionString = "Data Source=MyServerName;Initial Catalog=Mantis;Integrated Security=SSPI;";

		private static SqlConnection mantisConnection;
		private static SqlTransaction mantisTransaction;
		private static SqlConnection taskVisionConnection;

		private static Hashtable ProjectTaskVisionIdToMantisId = new Hashtable();
		private static Hashtable UserTaskVisionIdToMantisId = new Hashtable();

		/// <summary>
		/// A method that does the mapping from task vision priority codes to Mantis priority codes.
		/// This will need to be updated before the migration is triggered.
		/// 
		/// TODO: Change implementation to be configuration based.
		/// </summary>
		/// <param name="TaskVisionPriority">The task vision priority</param>
		/// <returns>The corresponding Mantis priority.</returns>
		private static int TaskVisionPriorityToMantisPriority(int TaskVisionPriority)
		{
			int priority;

			switch(TaskVisionPriority)
			{
				case 3: // minor
					priority = 20;
					break;
				case 1: // medium
					priority = 30;
					break;
				case 2: // major
					priority = 40;
					break;
				default:
					throw new NotSupportedException(String.Format("Unsupported priority {0}", TaskVisionPriority));
			}

			return priority;
		}

		/// <summary>
		/// A method that does the mapping from task vision status codes to Mantis status codes.
		/// This will need to be updated before the migration is triggered.
		/// 
		/// TODO: Change implementation to be configuration based.
		/// </summary>
		/// <param name="TaskVisionStatus">The task vision status code.</param>
		/// <returns>The corresponding Mantis status code.</returns>
		private static int TaskVisionStatusToMantisStatus(int TaskVisionStatus)
		{
			int status;
			switch(TaskVisionStatus)
			{
				case 1: // open
					status = 10;
					break;
				case 2: // closed
					status = 90;
					break;
				case 3: // deferred
					status = 80;
					break;
				case 4: // designing
					status = 20;
					break;
				case 5: // coding
					status = 30;
					break;
				case 6: // testing
					status = 40;
					break;
				case 7: // deploying
					status = 50;
					break;
				case 8: // releasing
					status = 70;
					break;
				case 9: // UAT
					status = 60;
					break;
				default:
					throw new NotSupportedException(String.Format("Unsupported status {0}.", TaskVisionStatus));
			}

			return status;
		}

		/// <summary>
		/// Import Task Vision Projects into Mantis
		/// </summary>
		private static void ImportProjects()
		{
			Console.WriteLine("Importing Projects...");

			SqlCommand sqlCmd = new SqlCommand("SELECT * FROM PROJECTS", taskVisionConnection);
			SqlDataReader dr = sqlCmd.ExecuteReader();

			while(dr.Read())
			{
				Console.WriteLine("  Project ID = {0}, Project Name = {1}, Project Description = {2}, IsDeleted = {3}, Date Created = {4}",
					dr["ProjectID"], dr["ProjectName"], dr["ProjectDescription"], dr["IsDeleted"], dr["DateCreated"]);

				SqlCommand sqlInsertCmd = new SqlCommand("INSERT INTO mantis_project_table " +
					"(name, status, enabled, view_state, description) VALUES " +
					"(@Name, 30, @Enabled, 10, @Description)", mantisConnection, mantisTransaction);

				SqlParameter sqlParam;

				sqlParam = new SqlParameter("@Name", SqlDbType.VarChar);
				sqlParam.Value = dr.GetString(dr.GetOrdinal("ProjectName"));
				sqlInsertCmd.Parameters.Add(sqlParam);

				sqlParam = new SqlParameter("@Description", SqlDbType.Text);
				sqlParam.Value = dr.GetString(dr.GetOrdinal("ProjectDescription"));
				sqlInsertCmd.Parameters.Add(sqlParam);

				sqlParam = new SqlParameter("@Enabled", SqlDbType.Bit);
				sqlParam.Value = !dr.GetBoolean(dr.GetOrdinal("IsDeleted"));
				sqlInsertCmd.Parameters.Add(sqlParam);

				sqlInsertCmd.ExecuteNonQuery();

				SqlCommand sqlGetIdCmd = new SqlCommand("SELECT @@IDENTITY", mantisConnection, mantisTransaction);
				object result = sqlGetIdCmd.ExecuteScalar();
				int insertID = Int32.Parse(result.ToString());

				ProjectTaskVisionIdToMantisId.Add(dr.GetInt32(dr.GetOrdinal("ProjectID")), insertID);
			}

			dr.Close();
		}

		/// <summary>
		/// Import Task Vision Users into Mantis
		/// </summary>
		private static void ImportUsers()
		{
			Console.WriteLine("Importing Users...");

			SqlCommand sqlCmd = new SqlCommand("SELECT * FROM USERS", taskVisionConnection);
			SqlDataReader dr = sqlCmd.ExecuteReader();

			while(dr.Read())
			{
				Console.WriteLine("  User ID = {0}, User Name = {1}, User Full Name = {2}, User Email = {3}, Is Account Locked = {4}, Is Administrator = {5}, Date Created = {6}",
					dr["UserID"], dr["UserName"], dr["UserFullName"], dr["UserEmail"], dr["IsAccountLocked"], dr["IsAdministrator"], dr["DateCreated"]);

				SqlCommand sqlInsertCmd = new SqlCommand("INSERT INTO mantis_user_table " +
					"(username, realname, email, password, date_created, enabled, protected, access_level, cookie_string) VALUES " +
					"(@userName, @realName, @email, @password, @dateCreated, @enabled, 0, @accessLevel, @cookieString)", mantisConnection, mantisTransaction);

				SqlParameter sqlParam;

				sqlParam = new SqlParameter("@userName", SqlDbType.VarChar);
				sqlParam.Value = dr.GetString(dr.GetOrdinal("UserName"));
				sqlInsertCmd.Parameters.Add(sqlParam);

				sqlParam = new SqlParameter("@realName", SqlDbType.VarChar);
				sqlParam.Value = dr.GetString(dr.GetOrdinal("UserFullName"));
				sqlInsertCmd.Parameters.Add(sqlParam);

				sqlParam = new SqlParameter("@email", SqlDbType.VarChar);
				sqlParam.Value = dr.GetString(dr.GetOrdinal("UserEmail"));
				sqlInsertCmd.Parameters.Add(sqlParam);

				sqlParam = new SqlParameter("@password", SqlDbType.VarChar);
				sqlParam.Value = "5f4dcc3b5aa765d61d8327deb882cf99";  // MD5 for "password"
				sqlInsertCmd.Parameters.Add(sqlParam);

				sqlParam = new SqlParameter("@dateCreated", SqlDbType.DateTime);
				sqlParam.Value = dr.GetDateTime(dr.GetOrdinal("DateCreated"));
				sqlInsertCmd.Parameters.Add(sqlParam);

				sqlParam = new SqlParameter("@enabled", SqlDbType.Bit);
				sqlParam.Value = !dr.GetBoolean(dr.GetOrdinal("IsAccountLocked"));
				sqlInsertCmd.Parameters.Add(sqlParam);

				sqlParam = new SqlParameter("@accessLevel", SqlDbType.Int);
				sqlParam.Value = dr.GetBoolean(dr.GetOrdinal("IsAdministrator")) ? /* admin */ 90 : /* developer */ 55;
				sqlInsertCmd.Parameters.Add(sqlParam);

				sqlParam = new SqlParameter("@cookieString", SqlDbType.VarChar);
				sqlParam.Value = dr["UserID"] + "-" + DateTime.Now.ToString();
				sqlInsertCmd.Parameters.Add(sqlParam);

				sqlInsertCmd.ExecuteNonQuery();

				SqlCommand sqlGetIdCmd = new SqlCommand("SELECT @@IDENTITY", mantisConnection, mantisTransaction);
				object result = sqlGetIdCmd.ExecuteScalar();
				int insertID = Int32.Parse(result.ToString());

				UserTaskVisionIdToMantisId.Add(dr.GetInt32(dr.GetOrdinal("UserID")), insertID);
			}

			dr.Close();
		}

		/// <summary>
		/// Import Task Vision Tasks into Mantis
		/// </summary>
		private static void ImportTasks()
		{
			Console.WriteLine("Importing Tasks...");

			SqlCommand sqlCmd = new SqlCommand("SELECT * FROM TASKS", taskVisionConnection);
			SqlDataReader dr = sqlCmd.ExecuteReader();

			while(dr.Read())
			{
				Console.WriteLine("  Task ID = {0}, Project ID = {1}, Modified By = {2}, Assigned To = {3}, Task Summary = {4}, Task Description = {5}, Priority Id = {6}, Status Id = {7}, Progress = {8}, IsDeleted = {9}, Date Due = {10}, Date Modified = {11}, Date Created = {12}",
					dr["TaskID"], dr["ProjectID"], dr["ModifiedBy"], dr["AssignedTo"], dr["TaskSummary"], "desc" /* dr["TaskDescription"]*/, dr["PriorityID"], dr["StatusID"], dr["Progress"], dr["IsDeleted"], dr["DateDue"], dr["DateModified"], dr["DateCreated"] );

				SqlCommand sqlInsertCmd;
				SqlParameter sqlParam;

				sqlInsertCmd = new SqlCommand("INSERT INTO mantis_bug_text_table " +
					"(description, steps_to_reproduce, additional_information) VALUES " +
					"(@description, '', @additionalInformation)", mantisConnection, mantisTransaction);

				sqlParam = new SqlParameter("@description", SqlDbType.Text);
				sqlParam.Value = dr.GetString(dr.GetOrdinal("TaskDescription"));
				sqlInsertCmd.Parameters.Add(sqlParam);

				sqlParam = new SqlParameter("@additionalInformation", SqlDbType.Text);
				sqlParam.Value = "Cross Reference to Task Vision: " + dr["TaskID"];
				sqlInsertCmd.Parameters.Add(sqlParam);

				sqlInsertCmd.ExecuteNonQuery();

				SqlCommand sqlGetIdCmd = new SqlCommand("SELECT @@IDENTITY", mantisConnection, mantisTransaction);
				object result = sqlGetIdCmd.ExecuteScalar();
				int bugTextInsertId = Int32.Parse(result.ToString());

				sqlInsertCmd = new SqlCommand("INSERT INTO mantis_bug_table " +
					"(project_id, reporter_id, handler_id, priority, severity, reproducibility, status, category, date_submitted, last_updated, bug_text_id, summary ) VALUES " +
					"(@projectId, @reporterId, @handlerId, @priority, @severity, 100, @status, 'General', @dateSubmitted, @lastUpdated, @bugTextId, @summary )", mantisConnection, mantisTransaction);

				sqlParam = new SqlParameter("@projectId", SqlDbType.Int);
				sqlParam.Value = ProjectTaskVisionIdToMantisId[dr.GetInt32(dr.GetOrdinal("ProjectId"))];
				sqlInsertCmd.Parameters.Add(sqlParam);

				sqlParam = new SqlParameter("@reporterId", SqlDbType.Int);
				sqlParam.Value = 1; // Administrator, task vision doesn't keep track of reported?! // UserTaskVisionIdToMantisId[dr.GetInt32(dr.GetOrdinal("AssignedTo"))];
				sqlInsertCmd.Parameters.Add(sqlParam);

				sqlParam = new SqlParameter("@handlerId", SqlDbType.Int);
				sqlParam.Value = UserTaskVisionIdToMantisId[dr.GetInt32(dr.GetOrdinal("AssignedTo"))];
				sqlInsertCmd.Parameters.Add(sqlParam);

				sqlParam = new SqlParameter("@bugTextId", SqlDbType.Int);
				sqlParam.Value = bugTextInsertId;
				sqlInsertCmd.Parameters.Add(sqlParam);

				sqlParam = new SqlParameter("@severity", SqlDbType.Int);
				sqlParam.Value = 40;  // medium
				sqlInsertCmd.Parameters.Add(sqlParam);

				sqlParam = new SqlParameter("@priority", SqlDbType.Int);
				sqlParam.Value = TaskVisionPriorityToMantisPriority(dr.GetInt32(dr.GetOrdinal("PriorityID")));
				sqlInsertCmd.Parameters.Add(sqlParam);

				sqlParam = new SqlParameter("@status", SqlDbType.Int);
				sqlParam.Value = TaskVisionStatusToMantisStatus(dr.GetInt32(dr.GetOrdinal("StatusID")));
				sqlInsertCmd.Parameters.Add(sqlParam);

				sqlParam = new SqlParameter("@dateSubmitted", SqlDbType.DateTime);
				sqlParam.Value = dr.GetDateTime(dr.GetOrdinal("DateCreated"));
				sqlInsertCmd.Parameters.Add(sqlParam);

				sqlParam = new SqlParameter("@lastUpdated", SqlDbType.DateTime);
				sqlParam.Value = dr.GetDateTime(dr.GetOrdinal("DateModified"));
				sqlInsertCmd.Parameters.Add(sqlParam);

				sqlParam = new SqlParameter("@summary", SqlDbType.VarChar, 128);
				sqlParam.Value = dr.GetString(dr.GetOrdinal("TaskSummary"));
				sqlInsertCmd.Parameters.Add(sqlParam);

				sqlInsertCmd.ExecuteNonQuery();

				result = sqlGetIdCmd.ExecuteScalar();
				int bugId = Int32.Parse(result.ToString());

				if (dr.GetBoolean(dr.GetOrdinal("IsDeleted")))
				{
					SqlCommand sqlDeleteCmd;

					// delete the mantis_bug_text_table record.
					sqlDeleteCmd = new SqlCommand(String.Format("DELETE FROM mantis_bug_text_table WHERE id = {0}", bugTextInsertId), mantisConnection, mantisTransaction);
					sqlDeleteCmd.ExecuteNonQuery();

					// delete the mantis_bug_table record.
					sqlDeleteCmd = new SqlCommand(String.Format("DELETE FROM mantis_bug_table WHERE id = {0}", bugId), mantisConnection, mantisTransaction);
					sqlDeleteCmd.ExecuteNonQuery();
				}
			}

			dr.Close();
		}

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string[] args)
		{
			try
			{
				using(taskVisionConnection = new SqlConnection(TaskVisionConnectionString))
				{
					using(mantisConnection = new SqlConnection(MantisConnectionString))
					{
						taskVisionConnection.Open();
						mantisConnection.Open();
						mantisTransaction = mantisConnection.BeginTransaction(IsolationLevel.Serializable);

						try
						{
							ImportProjects();
							ImportUsers();
							ImportTasks();
							mantisTransaction.Commit();
						}
						catch
						{
							mantisTransaction.Rollback();
						}
					}
				}
			}
			catch(Exception ex)
			{
				Console.WriteLine(ex.ToString());
			}
		}
	}
}
